﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using TouchSocket.Sockets;
using System.ComponentModel;
using TouchSocket.Core;


namespace Toolkit.Tcp
{
    public class BgCommunicate:INotifyPropertyChanged
    {
        private IWaitingClient<TouchSocket.Sockets.SocketClient> WaitClientServer;
        public TcpService TcpService { get; private set; }

        TouchSocketConfig Config;

        public event Action<SocketClient, ConnectedEventArgs> ConnectedEvent;
        public event Action<SocketClient, DisconnectEventArgs> DisconnectedEvent;
        public event Action<SocketClient, string> ReceivedEvent;
        public event Action<SocketClient, string> SendEvent;
        public event PropertyChangedEventHandler PropertyChanged;

       

        public int ClientCount { get => TcpService.Count; }

        public BgCommunicate()
        {
            Config = new TouchSocketConfig();
            TcpService = new TcpService();

            var config = new TouchSocketConfig();
            config.SetListenIPHosts(new IPHost[] { new IPHost(503) }); //同时监听两个地址//503

            config.SetTcpDataHandlingAdapter(() => { return new NormalDataHandlingAdapter(); });       ////亦或者省略\r\n，但此时调用方不能高速调用，会粘包
            //载入配置
            TcpService.Setup(config);

            /////有客户端成功连接
            TcpService.Connected = ConnectedVoid;
            //有客户端断开连接
            TcpService.Disconnected = DisconnectedVoid;
            //从客户端收到信息
            TcpService.Received = ReceivedVoid;
        }

        /// <summary>
        /// 启动监听
        /// </summary>
        public void StartListening()
        {
            TcpService.Start();
        }

        /// <summary>
        /// 停止监听
        /// </summary>
        public void StopListening()
        {
            TcpService.Stop();
        }

        public void Dispose()
        {
            TcpService.Stop();
            TcpService.Dispose();
        }

        #region 事件接收
        /// <summary>
        /// 有客户端成功连接
        /// </summary>
        /// <param name="client"></param>
        /// <param name="e"></param>
        /// <returns></returns>
        private Task ConnectedVoid(SocketClient client, ConnectedEventArgs e)
        {
            //调用CreateWaitingClient获取到IWaitingClient的对象。
            WaitClientServer = client.CreateWaitingClient(new WaitingOptions()
            {
                FilterFunc = response => //设置用于筛选的fun委托，当返回为true时，才会响应返回
                {
                    return true;

                    //if (response.Data.Length == 1)
                    //{
                    //    return true;
                    //}
                    //return false;
                }
            });
            ConnectedEvent?.Invoke(client, e);
            return EasyTask.CompletedTask;
        }
        /// <summary>
        /// 有客户端断开连接
        /// </summary>
        /// <param name="client"></param>
        /// <param name="e"></param>
        /// <returns></returns>
        private Task DisconnectedVoid(SocketClient client, DisconnectEventArgs e)
        {
            DisconnectedEvent?.Invoke(client, e);
            if ( WaitClientServer != null )
            {
                if ( WaitClientServer.Client.Id == client.Id )
                {
                    if ( TcpService.Count > 0 )
                    {
                        WaitClientServer = ( ( SocketClient ) ( TcpService.SocketClients.GetClients().ElementAt(0) ) ).CreateWaitingClient(new WaitingOptions()
                        {
                            FilterFunc = response => //设置用于筛选的fun委托，当返回为true时，才会响应返回
                            {
                                return true;

                                //if (response.Data.Length == 1)
                                //{
                                //    return true;
                                //}
                                //return false;
                            }
                        });
                    }
                }
            }
            return EasyTask.CompletedTask;
        }
        /// <summary>
        /// 从客户端收到信息
        /// </summary>
        /// <param name="client"></param>
        /// <param name="e"></param>
        /// <returns></returns>
        private Task ReceivedVoid(SocketClient client, ReceivedDataEventArgs e)
        {
            Encoding encoding = GetEncoding();
            ReceivedEvent?.Invoke(client, encoding.GetString(e.ByteBlock.Buffer, 0, e.ByteBlock.Len));
            //调用CreateWaitingClient获取到IWaitingClient的对象。
            WaitClientServer = client.CreateWaitingClient(new WaitingOptions()
            {
                FilterFunc = response => //设置用于筛选的fun委托，当返回为true时，才会响应返回
                {
                    return true;

                    //if (response.Data.Length == 1)
                    //{
                    //    return true;
                    //}
                    //return false;
                }
            });
            return EasyTask.CompletedTask;
        }
        #endregion

        #region 发送数据
        /// <summary>
        /// 发送信息给特定ID的客户端
        /// </summary>
        /// <param name="ID"></param>
        /// <param name="msg"></param>
        public void Send(string ID, string mes)
        {
            Encoding encoding = GetEncoding();
            var clients = TcpService.GetClients();
            var client = TcpService.GetClients().FirstOrDefault(cl => cl.Id == ID);
            if ( client != null )
            {
                client.Send(encoding.GetBytes(mes));
                SendEvent?.Invoke(client, mes);
            }
        }

        /// <summary>
        /// 发送信息给特定ID的客户端
        /// </summary>
        /// <param name="ID"></param>
        /// <param name="mes"></param>
        public async Task SendAsync(string ID, string mes)
        {
            Encoding encoding = GetEncoding();
            var clients = TcpService.GetClients();
            var client = TcpService.GetClients().FirstOrDefault(cl => cl.Id == ID);
            if ( client != null )
            {
                await client.SendAsync(encoding.GetBytes(mes));
                SendEvent?.Invoke(client, mes);
            }
        }

        /// <summary>
        /// 发送信息给在线的所有客户端
        /// </summary>
        /// <param name="ID"></param>
        /// <param name="msg"></param>
        public void Send(string mes)
        {
            Encoding encoding = GetEncoding();
            foreach ( var client in TcpService.GetClients() )
            {
                client.Send(encoding.GetBytes(mes));
                SendEvent?.Invoke(client, mes);
            }
        }

        /// <summary>
        /// 发送信息给在线的所有客户端
        /// </summary>
        /// <param name="mes"></param>
        public async Task SendAsync(string mes)
        {
            Encoding encoding = GetEncoding();
            foreach ( var client in TcpService.GetClients() )
            {
                await client.SendAsync(encoding.GetBytes(mes));
                SendEvent?.Invoke(client, mes);
            }
        }

        /// <summary>
        /// 发送信息给客户端
        /// </summary>
        /// <param name="client">客户端</param>
        /// <param name="mes"></param>
        public void Send(SocketClient client, string mes)
        {
            if ( client == null ) return;
            if ( !client.Online ) return;
            Encoding encoding = GetEncoding();
            client.Send(encoding.GetBytes(mes));
            SendEvent?.Invoke(client, mes);
        }

        /// <summary>
        /// 发送信息给客户端
        /// </summary>
        /// <param name="client"></param>
        /// <param name="mes"></param>
        public async Task SendAsync(SocketClient client, string mes)
        {
            if ( client == null ) return;
            if ( !client.Online ) return;
            Encoding encoding = GetEncoding();
            await client.SendAsync(encoding.GetBytes(mes));
            SendEvent?.Invoke(client, mes);
        }

        /// <summary>
        /// 发送并接收数据(最后一次活动的客户端)
        /// </summary>
        /// <param name="mes"></param>
        /// <param name="millisecondsTimeout"></param>
        /// <returns></returns>
        public string SendThenReturn(string mes, int millisecondsTimeout = 5000)
        {
            Encoding encoding = GetEncoding();
            SendEvent?.Invoke(WaitClientServer.Client, mes);
            var returnData = WaitClientServer.SendThenReturn(encoding.GetBytes(mes), millisecondsTimeout);
            ReceivedEvent?.Invoke(WaitClientServer.Client, encoding.GetString(returnData));
            return encoding.GetString(returnData);
        }

        /// <summary>
        /// 发送并接收数据(最后一次活动的客户端)
        /// </summary>
        /// <param name="mes"></param>
        /// <param name="millisecondsTimeout"></param>
        /// <returns></returns>
        public async Task<string> SendThenReturnAsync(string mes, int millisecondsTimeout = 5000)
        {
            Encoding encoding = GetEncoding();
            SendEvent?.Invoke(WaitClientServer.Client, mes);
            var returnData = await WaitClientServer.SendThenReturnAsync(encoding.GetBytes(mes), millisecondsTimeout);
            ReceivedEvent?.Invoke(WaitClientServer.Client, encoding.GetString(returnData));
            return encoding.GetString(returnData);
        }

        /// <summary>
        /// 发送并接收数据
        /// </summary>
        /// <param name="client"></param>
        /// <param name="mes"></param>
        /// <param name="millisecondsTimeout"></param>
        /// <returns></returns>
        public string SendThenReturn(SocketClient client, string mes, int millisecondsTimeout = 5000)
        {
            WaitClientServer = client.CreateWaitingClient(new WaitingOptions()
            {
                FilterFunc = response => //设置用于筛选的fun委托，当返回为true时，才会响应返回
                {
                    return true;

                    //if (response.Data.Length == 1)
                    //{
                    //    return true;
                    //}
                    //return false;
                }
            });
            Encoding encoding = GetEncoding();
            SendEvent?.Invoke(WaitClientServer.Client, mes);
            var returnData = WaitClientServer.SendThenReturn(encoding.GetBytes(mes), millisecondsTimeout);
            ReceivedEvent?.Invoke(WaitClientServer.Client, encoding.GetString(returnData));
            return encoding.GetString(returnData);
        }

        /// <summary>
        /// 发送并接收数据
        /// </summary>
        /// <param name="client"></param>
        /// <param name="mes"></param>
        /// <param name="millisecondsTimeout"></param>
        /// <returns></returns>
        public async Task<string> SendThenReturnAsync(SocketClient client, string mes, int millisecondsTimeout = 5000)
        {
            WaitClientServer = client.CreateWaitingClient(new WaitingOptions()
            {
                FilterFunc = response => //设置用于筛选的fun委托，当返回为true时，才会响应返回
                {
                    return true;

                    //if (response.Data.Length == 1)
                    //{
                    //    return true;
                    //}
                    //return false;
                }
            });
            Encoding encoding = GetEncoding();
            SendEvent?.Invoke(WaitClientServer.Client, mes);
            var returnData = await WaitClientServer.SendThenReturnAsync(encoding.GetBytes(mes), millisecondsTimeout);
            ReceivedEvent?.Invoke(WaitClientServer.Client, encoding.GetString(returnData));
            return encoding.GetString(returnData);
        }
        #endregion

        #region 编码
        /// <summary>
        /// 获取编码
        /// </summary>
        /// <param name="dataEncoding"></param>
        /// <returns></returns>
        public Encoding GetEncoding()
        {
           
         
            return  Encoding.Unicode;
        }
        #endregion
    }
}
